<?php
/*
Plugin Name: WP-Nicodo
Plugin URI: http://akabeko.sakura.ne.jp/blog/software/wp-nicodo/
Description: ニコニコ動画のコンテンツ情報をページに埋め込みます。
Version: 1.0
Author: アカベコ
Author URI: http://akabeko.sakura.ne.jp/
*/

/*  Copyright 2009 Akabeko

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// プラグインのインスタンス生成
if( class_exists( 'WpNicodo' ) )
{
	$wpnicodo = new WpNicodo();
}

/**
 * プラグインの処理を行います。
 */
class WpNicodo
{
	/**
	 * WordPress のデータベースに登録するプラグインの設定名。
	 */
	private $optionName;

	/**
	 * プラグインの設定を格納する連想配列。
	 */
	private $options;

	/**
	 * プラグインの配置されたディレクトリを示す URL。
	 */
	private $pluginDirUrl;

	/**
	 * インスタンスを初期化します。
	 */
	public function WpNicodo()
	{
		$this->pluginDirUrl = WP_PLUGIN_URL . '/' . array_pop( explode( DIRECTORY_SEPARATOR, dirname( __FILE__ ) ) );
		$this->optionName   = 'wp_nicodo_options';
		$this->options      = $this->getOption();

		// ハンドラの登録
		add_action( 'admin_head', array( &$this, 'onAdminHead' ) );
		add_action( 'admin_menu', array( &$this, 'onAdminMenu' ) );
		add_action( 'wp_head',    array( &$this, 'onWpHead'    ) );
		add_shortcode( 'nicodo', array( &$this, 'onExecuteShortCode' ) );
	}

	/**
	 * 記事に挿入する為のニコニコ動画情報を取得します。
	 *
	 * @param	$id	動画の ID。
	 *
	 * @return	ニコニコ動画情報。
	 */
	private function getNicoInfoDefault( $id )
	{
		return '<p><iframe width="320" height="196" src="http://ext.nicovideo.jp/thumb/'. $id .'" scrolling="no" style="border:solid 1px #AAA;" frameborder="0"></iframe></p>';
	}

	/**
	 * 記事に挿入する為のテンプレート版ニコニコ動画情報を取得します。
	 * このメソッドはテンプレートとして登録されている HTML 内のキーワードを、適切な動画情報に置き換えた HTML を返します。
	 *
	 * @param	$id	動画の ID。
	 *
	 * @return	テンプレート版のニコニコ動画情報。
	 */
	private function getNicoInfoTemplate( $id )
	{
		$newTemplate = $this->options[ 'template' ];
		if( $newTemplate == '' ) { return '<p>Error : Not defined template.</p>'; }

		$xml = simplexml_load_file( 'http://www.nicovideo.jp/api/getthumbinfo/' . $id );
		if( !$xml ) { return '<p>Error : Not found video.</p>'; }

		// 単一のデータはそのまま置き換え
		$newTemplate = str_replace( '[video_id]',       $xml->thumb->video_id,      $newTemplate );
		$newTemplate = str_replace( '[title]',          $xml->thumb->title,         $newTemplate );
		$newTemplate = str_replace( '[thumbnail_url]',  $xml->thumb->thumbnail_url, $newTemplate );
		$newTemplate = str_replace( '[length]',         $xml->thumb->length,        $newTemplate );
		$newTemplate = str_replace( '[last_res_body]',  $xml->thumb->last_res_body, $newTemplate );
		$newTemplate = str_replace( '[watch_url]',      $xml->thumb->watch_url,     $newTemplate );
		$newTemplate = str_replace( '[thumb_type]',     $xml->thumb->thumb_type,    $newTemplate );

		// 数値は千の位ごとにカンマを付ける
		$newTemplate = str_replace( '[view_counter]',   number_format( $xml->thumb->view_counter ),   $newTemplate );
		$newTemplate = str_replace( '[comment_num]',    number_format( $xml->thumb->comment_num ),    $newTemplate );
		$newTemplate = str_replace( '[mylist_counter]', number_format( $xml->thumb->mylist_counter ), $newTemplate );

		// 日時は yyyy/mm/dd hh:ss 形式に変換 ( iframe 版の場合、年が 2 桁だが、分かりにくいので 4 桁にしておく )
		$newTemplate = str_replace( '[first_retrieve]', date( 'Y/m/d H:i', strtotime( $xml->thumb->first_retrieve ) ),  $newTemplate );

		// コメントは 100 文字以降を切る
		$newTemplate = str_replace( '[description]',  mb_strimwidth( $xml->thumb->description, 0, 99, '...', 'utf-8' ),  $newTemplate );

		// タグは繰り返し置換によってリストアップを行う
		if( preg_match( '|\[tags/\](.+?)\[/tags\]|s', $newTemplate, $match ) )
		{
			$max = count( $xml->thumb->tags->tag );
			$tags = "";

			for( $i = 0; $i < $max; ++$i )
			{
				$tags .= str_replace( '[value]', $xml->thumb->tags->tag[ $i ], $match[ 1 ] );
			}

			$tags        = str_replace( $match[ 1 ], $tags, $match[ 0 ]  );
			$tags        = str_replace( '[tags/]',   '',    $tags        );
			$tags        = str_replace( '[/tags]',   '',    $tags        );
			$newTemplate = str_replace( $match[ 0 ], $tags, $newTemplate );
		}

		return $newTemplate;
	}

	/**
	 * プラグインの設定を取得します。
	 *
	 * @return	設定を格納した連想配列。
	 */
	private function getOption()
	{
		$options = get_option( $this->optionName );
		return ( is_array( $options ) ? $options : $this->getOptionDefalt() );
	}

	/**
	 * プラグインのデフォルト設定を取得します。
	 *
	 * @return	設定を格納した連想配列。
	 */
	private function getOptionDefalt()
	{
		$options = array(
			'display' => 'default',
			'css' => $this->pluginDirUrl . '/nicodo.css',
			'template' => '<div id="nicodo">
	<table>
		<tr><th colspan="2"><a href="http://www.nicovideo.jp/" target="_blank">ニコニコ動画</a> [thumb_type]</th></tr>
		<tr><td colspan="2" class="counter">再生 : <strong>[view_counter]</strong> コメント : <strong>[comment_num]</strong> マイリスト : <strong>[mylist_counter]</strong></td></tr>
		<tr valign="top"><td><img src="[thumbnail_url]"><br /><strong>[length]</strong></td><td class="detail"><strong>[first_retrieve]</strong> 投稿<br /><strong><a href="[watch_url]" target="blank">[title]</a></strong><br />[description]</td></tr>
		<tr><td colspan="2"><p class="res">[last_res_body]</p></td></tr>
	</table>
</div>'
		);

		return $options;
	}

	/**
	 * 管理画面のヘッダー部分が設定される時に発生します。
	 */
	public function onAdminHead()
	{
		//$dir = array_pop( explode( DIRECTORY_SEPARATOR, dirname( __FILE__ ) ) );
		//$cssdir = WP_PLUGIN_DIR . '/' . $dir;
?>
<link rel="stylesheet" type="text/css" href="<?php echo $this->pluginDirUrl; ?>/admin.css" />
<?php
	}

	/**
	 * 管理画面が設定される時に発生します。
	 */
	public function onAdminMenu()
	{
		// オプションページの追加
		add_options_page( 'WP-Nicodo の設定', 'WP-Nicodo', 8, basename(__FILE__), array( &$this, 'onOptionPage' ) ) ;
	}

	/**
	 * ショートコードが実行される時に発生します。
	 *
	 * @param	$atts		ショートコードに指定されたパラメータのコレクション。
	 * @param	$content	ショートコードのタグに囲まれたコンテンツ。
	 *
	 * @return	ショートコードの実行結果。
	 */
	public function onExecuteShortCode( $atts, $content )
	{
		extract( shortcode_atts( array( 'display' => '' ), $atts ) );

		switch( $display )
		{
		case 'template':
			return $this->getNicoInfoTemplate( $content );

		case 'default':
			return $this->getNicoInfoDefault( $content );

		default:
			// 表示モードが指定されていない場合は、既定のモードで表示
			return ( $this->options[ 'display' ] == "template" ? $this->getNicoInfoTemplate( $content ) : $this->getNicoInfoDefault( $content ) );
		}
	}

	/**
	 * プラグインの設定ページが表示される時に発生します。
	 */
	public function onOptionPage()
	{
		if( $_POST[ 'option' ] == 'update' )
		{
			$this->options[ 'template' ] = stripslashes( $_POST[ 'template' ] );
			$this->options[ 'display'  ] = $_POST[ 'display' ];
			$this->options[ 'css'      ] = $_POST[ 'css'     ];
			$this->options[ 'use_css'  ] = $_POST[ 'useCss'  ];


			update_option( $this->optionName, $this->options );
		}

		// ラジオボタンの既定選択
		$checked         = 'checked="checked"';
		$selectedDisplay = array();
		switch( $this->options[ 'display' ] )
		{
		case 'template':
			$selectedDisplay[ 'template' ] = $checked;
			break;

		default:
			$selectedDisplay[ 'default' ] = $checked;
			break;
		}

		$selectedUseCss = ( $this->options[ 'use_css' ] == 'true' ? $checked : '' );
?>
	<h2>WP-Nicodo の設定</h2>
	<div id="wpnicodo">
		<form action="<?php echo $_SERVER[ 'REQUEST_URI' ]; ?>" method="post">
			<fieldset>
				<legend>表示方法</legend>
				<ul>
					<li><input type="radio" name="display" id="displayDefault" value="default" <?php echo $selectedDisplay[ "default" ]; ?> /><label for="displayDefault">ニコニコ動画の標準フレーム ( デフォルト )</label></li>
					<li><input type="radio" name="display" id="displayTemplate" value="template" <?php echo $selectedDisplay[ "template" ]; ?> /><label for="displayTemplate">テンプレート</label></li>
				</ul>
			</fieldset>
			<fieldset>
				<legend>スタイルシート</legend>
				<p>
				スタイルシートを使用する場合は、以下のチェックボックスを有効にしてスタイルシートの URL を設定して下さい。<br />
				これらの設定を行う事で、記事を表示した時にスタイルシートが読み込まれます。
				</p>
				<input type="checkbox" name="useCss" value="true" <?php echo $selectedUseCss; ?> />スタイルシートを使用する<br />
				<input type="text" name="css"  size="100" value="<?php echo $this->options[ 'css' ] ?>" />
			</fieldset>
			<fieldset>
				<legend>テンプレート</legend>
				<p>
				ニコニコ動画の情報を表示する為の HTML テンプレートです。<br />
				特定のキーワードがニコニコ動画の情報に置き換えられるので、HTML と組み合わせて下さい。
				</p>
				<textarea name="template" rows="30" cols="100"><?php echo htmlspecialchars( $this->options[ "template" ] ); ?></textarea>

			</fieldset>
			<fieldset>
				<legend>テンプレートに使用できるキーワード</legend>
				<p>
				以下のキーワードを指定すると、対応する動画情報へ置き換えられます。<br />
				キーワード名は基本的に「ニコニコ動画 API」の getthumbinfo から取得される XML のタグ名に準拠しています。
				</p>
				<table>
					<thead>
						<tr><th>キーワード</th><th>対応する動画の情報</th></tr>
					</thead>
					<tbody>
						<tr><td>[video_id]</td><td>動画の ID。sm123456 のような書式となります。</td></tr>
						<tr><td>[title]</td><td>動画のタイトル。</td></tr>
						<tr><td>[description]</td><td>動画の説明。</td></tr>
						<tr><td>[thumbnail_url]</td><td>動画のサムネイル画像への URL。img タグの src 属性などに指定します。</td></tr>
						<tr><td>[first_retrieve]</td><td>投稿日時。</td></tr>
						<tr><td>[length]</td><td>動画の長さ ( 再生時間 )。</td></tr>
						<tr><td>[view_counter]</td><td>再生数。</td></tr>
						<tr><td>[comment_num]</td><td>コメント数。</td></tr>
						<tr><td>[mylist_counter]</td><td>マイリスト数。</td></tr>
						<tr><td>[last_res_body]</td><td>ブログパーツ表示用の最新コメント。</td></tr>
						<tr><td>[watch_url]</td><td>視聴用の URL。ニコニコ動画へのリンクを貼る場合は、この URL を使用します。</td></tr>
						<tr><td>[thumb_type]</td><td>動画なら「video」、マイメモリーなら「mymemory」となります。</td></tr>
						<tr>
							<td>[tags/] ～ [value] ～ [/tags]</td>
							<td>
								動画に付けられた全てのタグ。タグの存在する数だけ [tags/] ～ [/tags] の間の [value] を置き換えます。
								例えば以下のように指定すると、タグのリストが出力されます。
								<pre>&lt;ul&gt;
    [tags/]&lt;li&gt;[value]&lt;/li&gt;[/tags]
&lt;/ul&gt;</pre>
							</td>
						</tr>
					</tbody>
				</table>
			</fieldset>

			<p class="submit">
				<input type="submit" value="設定を更新する" />
				<input class="button-primary" type="hidden" name="option" value="update" />
			</p>
		</form>
	</div>
<?php
	}

	/**
	 * WordPress のヘッダ出力が行われる時に発生します。
	 */
	public function onWpHead()
	{
		if( $this->options[ 'use_css' ] == 'true' )
		{
			print( '<link rel="stylesheet" href="' . $this->options[ 'css' ] . '" type="text/css" media="screen" />' );
		}
	}
}

?>